( cd usr/lib ; rm -rf libprotoc.so )
( cd usr/lib ; ln -sf libprotoc.so.4.0.0 libprotoc.so )
( cd usr/lib ; rm -rf libprotoc.so.4 )
( cd usr/lib ; ln -sf libprotoc.so.4.0.0 libprotoc.so.4 )
( cd usr/lib ; rm -rf libprotobuf.so.4 )
( cd usr/lib ; ln -sf libprotobuf.so.4.0.0 libprotobuf.so.4 )
( cd usr/lib ; rm -rf libprotobuf.so )
( cd usr/lib ; ln -sf libprotobuf.so.4.0.0 libprotobuf.so )
